/*-
 * Copyright (c) 2016 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "vre2.h"

#define CATCHALL                                \
        catch (const exception& ex) {           \
                return ex.what();               \
        }                                       \
        catch (...) {                           \
                return "Unknown error";         \
        }

using namespace std;

vre2::vre2(const char *pattern) {
        re_ = new RE2(pattern);
        if (!re_->ok())
                throw new runtime_error(re_->error() + " at '"
                    + re_->error_arg() + "'");
}

vre2::~vre2() {
        if (re_) {
                delete re_;
                re_ = NULL;
        }
}

bool
vre2::match(const char *subject)
{
        return RE2::PartialMatchN(subject, *re_, NULL, 0);
}

const char *
vre2_init(vre2 **vre2p, const char *pattern)
{
        try {
                *vre2p = new vre2(pattern);
                return NULL;
        }
        CATCHALL
}

const char *
vre2_match(vre2 *vre2, const char *subject, int *match)
{
        try {
                *match = vre2->match(subject);
                return NULL;
        }
        CATCHALL
}

const char *
vre2_fini(vre2 **vre2)
{
        try {
                delete *vre2;
                return NULL;
        }
        CATCHALL
}
